/*
	(c) 2021 Dmitry Grinberg   https://dmitry.gr
	Non-commercial use only OR licensing@dmitry.gr
*/

#include <string.h>
#include "printf.h"
#include "util.h"
#include "vfs.h"



static void showProgress(uint32_t done, uint32_t of)
{
	pr("\r  0x%08x / 0x%08x -> %3u%%", done, of, done * 100 / of);
}

bool copyWithProgress(void *dst, struct VfsFile *src, uint32_t len)
{
	uint32_t j, now;

	for (j = 0; j < len; j += now) {
		
		now = len - j;
		if (now > 32768)
			now = 32768;
		showProgress(j, len);
				
		if (now != vfsFileRead(src, ((char*)dst) + j, now))
			return false;
	}
	showProgress(j, len);
	pr("\n");
	
	return true;
}

void zeroWithProgress(void *dst, uint32_t len)
{
	uint32_t j, now;

	for (j = 0; j < len; j += now) {
		
		now = len - j;
		if (now > 32768)
			now = 32768;
		showProgress( j, len);
		memset(((char*)dst) + j, 0, now);
	}
	showProgress(j, len);
	pr("\n");
}

int strcasecmp(const char *s1, const char *s2)
{
	char c1, c2;
	
	do{
		c1 = *s1++;
		c2 = *s2++;
		
		if (c1 >= 'a' && c1 <= 'z')
			c1 += 'A' - 'a';
		if (c2 >= 'a' && c2 <= 'z')
			c2 += 'A' - 'a';
		
	} while (c1 && c2 && c1 == c2);
		
	return c1 - c2;
}