/*
	(c) 2021 Dmitry Grinberg   https://dmitry.gr
	Non-commercial use only OR licensing@dmitry.gr
*/

#include  <stdint.h>
#define VEC_(nm, pfx)	void nm##pfx(void) __attribute__ ((weak, alias ("IntDefaultHandler"))) 
#define VEC(nm)		VEC_(nm, _Handler)
#define VECI(nm)	VEC_(nm, _IRQHandler)


void __attribute__ ((weak)) IntDefaultHandler(void);
VEC(NMI);
VEC(HardFault);
VEC(SVC);
VEC(PendSV);
VEC(SysTick);

VECI(PM);
VECI(SYSCTRL);
VECI(WDT);
VECI(RTC);
VECI(EIC);
VECI(NVMCTRL);
VECI(DMAC);
VECI(USB);
VECI(EVSYS);
VECI(SERCOM0);
VECI(SERCOM1);
VECI(SERCOM2);
VECI(SERCOM3);
VECI(SERCOM4);
VECI(SERCOM5);
VECI(TCC0);
VECI(TCC1);
VECI(TCC2);
VECI(TC3);
VECI(TC4);
VECI(TC5);
VECI(TC6);
VECI(TC7);
VECI(ADC);
VECI(AC);
VECI(DAC);
VECI(PTC);
VECI(I2S);





//main must exist
extern void micromain(void);

//stack top (provided by linker)
extern void __stack_top();
extern uint32_t __data_data[];
extern uint32_t __data_start[];
extern uint32_t __data_end[];
extern uint32_t __bss_start[];
extern uint32_t __bss_end[];




void __attribute__((noreturn)) IntDefaultHandler(void)
{
	while (1) {		
		asm("wfi":::"memory");
	}
}

void __attribute__((noreturn, naked)) ResetISR(void)
{
	asm volatile(
		"	adr		r7, 9f					\n\t"
		"	ldmia	r7!, {r0-r5}			\n\t"
		"	movs	r6, #0					\n\t"
		"	mov		sp, r5					\n\t"
		
		"1:									\n\t"
		"	cmp		r0, r2					\n\t"
		"	beq		2f						\n\t"
		"	ldmia	r1!, {r5}				\n\t"
		"	stmia	r0!, {r5}				\n\t"
		"	b		1b						\n\t"
		
		"2:									\n\t"
		"	cmp		r3, r4					\n\t"
		"	beq		3f						\n\t"
		"	stmia	r3!, {r6}				\n\t"
		"	b		2b						\n\t"
		
		"3:									\n\t"
		"	bl		micromain				\n\t"
		
		
		".balign 4							\n\t"
		"9:									\n\t"
		"	.word __data_start				\n\t"
		"	.word __data_data				\n\t"
		"	.word __data_end				\n\t"
		"	.word __bss_start				\n\t"
		"	.word __bss_end					\n\t"
		"	.word __stack_top				\n\t"
	);
}


__attribute__ ((section(".vectors"))) void (*const __VECTORS[]) (void) =
{
	(void*)0x20000f00,	//must be less than 8K due to old bootloader bug. crt will load proper value
	ResetISR,
	NMI_Handler,
	HardFault_Handler,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	SVC_Handler,		// SVCall handler
	0,					// Reserved
	0,					// Reserved
	PendSV_Handler,		// The PendSV handler
	SysTick_Handler,	// The SysTick handler
	
	// Chip Level - ATSAMD21
	PM_IRQHandler,
	SYSCTRL_IRQHandler,
	WDT_IRQHandler,
	RTC_IRQHandler,
	EIC_IRQHandler,
	NVMCTRL_IRQHandler,
	DMAC_IRQHandler,
	USB_IRQHandler,
	EVSYS_IRQHandler,
	SERCOM0_IRQHandler,
	SERCOM1_IRQHandler,
	SERCOM2_IRQHandler,
	SERCOM3_IRQHandler,
	SERCOM4_IRQHandler,
	SERCOM5_IRQHandler,
	TCC0_IRQHandler,
	TCC1_IRQHandler,
	TCC2_IRQHandler,
	TC3_IRQHandler,
	TC4_IRQHandler,
	TC5_IRQHandler,
	TC6_IRQHandler,
	TC7_IRQHandler,
	ADC_IRQHandler,
	AC_IRQHandler,
	DAC_IRQHandler,
	PTC_IRQHandler,
	I2S_IRQHandler,
};



